function [ber_demod] = mil110CSimulador(EsNodB, seed)
%----------------------------------------------
%Estrutura Geral do Simulador MIL-STD-188-110C:
%----------------------------------------------
%TRANSMISSOR: (bits) --> CODIFICADOR CONVOLUCIONAL --> INTERLEAVER --> MAPEADOR GRAY --> (SCRAMBLER + INSERCAO DE PREAMBULO/MINIPROBES) --> MODULADOR 8-PSK --> FORMATADOR PULSO --> (entrada do canal)
%
%CANAL HF: (saida do transmissor) --> MODELO WATTERSON --> RUIDO AWGN --> (entrada do receptor)
%
%RECEPTOR HARD: (saida do canal) --> (filtro recepcao)  --> (sincronizador quadro/simbolo) --> (equalizacao) --> (REMOCAO DE PREAMBULO/MINIPROBES + DESCRAMBLER) --> DEMODULADOR --> DEMAPEADOR GRAY --> DEINTERLEAVER --> (entrada do decodificador 'hard')
%
%RECEPTOR SOFT: (saida do canal) --> (filtro recepcao)  --> (sincronizador quadro/simbolo) --> (equalizacao) --> (REMOCAO DE PREAMBULO/MINIPROBES + DESCRAMBLER) --> (formacao de 'soft' bits) --> DEINTERLEAVER --> (entrada do decodificador 'soft')

%-----------------
%MIL-STD-188-110C:
%-----------------
%Variavel que ajusta o tamanho (duracao) do bloco de interleaving (entrelacamento).
%Opcoes sao de 0.6s (interleaving curto) e 4.8s (interleaving longo);
varMil110C.interLength = 4.8;

%Variavel que ajusta o numero de blocos de interleaving transmitidos;
varMil110C.nFrames     = 1;

%Variavel correspondente a semente que controla o gerador aleatorio
%utilizado pelo MATLAB. Esta funcionalidade permite garantir que toda
%transmissao independente (realizacao), empregando as mesmas variaveis de
%ajuste e a mesma semente, gere exatamente o mesmo resultado.
varMil110C.seed        = seed;

%--------------------------
%Codificador convolucional:
%--------------------------
% O codificador convolucional utilizado por este simulador corresponde a um
%codigo proprietario. Entretanto, sua funcionalidade e equivalente a do
%objeto 'comm.convolutionalEncoder' do MATLAB. Este objeto pode ser chamado
%da sequinte maneira:
%
% h_conv_cod = comm.ConvolutionalEncoder(poly2trellis(7,[133 171]),'TerminationMethod','Truncated'),
%
%onde '7' e o constraint length (numero de registros de memoria do codificador-
%mae, incluindo o do bit de entrada no instante atual) e '[133 171]' representam
%os 2 polinomios geradores do codigo '1/2', em octal. Em seguida, os bits
%de entrada podem ser codificados fazendo
%
% CodOutInterIn = step(h_conv_cod, BitInCodIn).

%-------
%Filtro:
%-------
%Variavel que ajusta o fator de rolloff do filtro formatador de pulso no
%transmissor;
varShapping.rolloffFactor      = 0.35;

%Variavel que ajusta o atraso (em simbolos) do filtro formatador de pulso;
varShapping.pulseDelay         = 5;

%Variavel que ajusta o fator de superamostragem implementado pelo filtro
%formatador de pulso. Retornara um erro se o valor adotado nao for tal que
%o produto 'RER x 2400 x oversamplingFactor' resulte um inteiro (sendo RER e
%o atraso (em segundos) entre os raios do canal Watterson);
varShapping.oversamplingFactor = 10;

%-----------------
%Watterson + AWGN:
%-----------------
%Variavel que seleciona o modelo de canal Watterson. Sao 4 opcoes, cada uma
%equivalendo a pares de valores de espalhamento de atraso (RER) e
%espalhamento Doppler (ED) distintos: canal 'bom' (RER = 0.0005s e ED =
%0.1Hz), canal 'moderado' (RER = 0.001s e ED = 0.5Hz), canal 'ruim' (RER =
%0.002s e ED = 1Hz) e canal 'muito_ruim' (RER = 0.005s e ED = 5Hz);
varWatterson.model  = 'ruim';

%Variavel que ajusta a razao sinal-ruido, em decibeis. Corresponde a razao
%entre a energia media do simbolo PSK transmitido (Es) e a densidade
%espectral de potencia do ruido (No);
varWatterson.EsNodB = EsNodB;

%Variavel correspondente a semente que controla o gerador aleatorio
%utilizado pelo MATLAB. Esta funcionalidade permite garantir que toda
%transmissao independente (realizacao), empregando as mesmas variaveis de
%ajuste e a mesma semente, gere exatamente o mesmo resultado.
varWatterson.seed   = seed;

%----------
%Simulador:
%----------
%mil110CParametros = carrega parametros gerais da forma de onda selecionada.
%Retorna um struct 'mil110C' que orienta o funcionamento dos demais componentes
%da cadeia de transmissao:
mil110C          = mil110CParametros(varMil110C.interLength);

%mil110CTransmissor = simula o transmissor MIL-STD-188-110C. Retorna uma struct
%'TxOut' com os seguintes vetores:
%
%BitInCodIn = bits de dados transmitidos. Entrada do codificador
%convolucional. Para quadros de interleaving curto (duração 0.6s), será um
%vetor de multiplos de 1440 bits; para quadros de interleaving longo
%(duração 4.8s), sera de multiplos de 11520 bits;
%
%CodOutInterIn = bits de saida do codificador convolucional e de entrada
%do interleaver. Como o codigo convolucional utiliza taxa 1/2, o tamanho
%deste vetor sera o dobro do vetor 'BitInCodIn' (multiplos de 2880 para
%interleaving curto, e multiplos de 11520 para interleaving longo);
%
%InterOutMapIn = bits de saida do interleaver e de entrada do mapeador
%Gray. Seu tamanho é o mesmo do vetor 'CodOutInterIn';
%
%MapOutMultiplexIn = simbolos PSK, inteiros, de saida do mapeador Gray e de
%entrada do formador de quadro (multiplexador). O tamanho deste vetor
%corresponde ao do vetor 'InterOutMapIn' dividido pelo numero de bits por
%simbolo. Como a forma de onda 2400bps utiliza modulação 8-PSK, seu tamanho
%será um terço da de 'InterOutMapIn';
%
%ComplexPSKSymbol = corresponde a versao modulada, em simbolos PSK
%complexos, do 'MapOutMultiplexIn'. Seu tamanho será o mesmo da do
%vetor 'MapOutMultiplexIn';
%
%ModConstellation = constelacao PSK utilizada para modular os simbolos de
%entrada do modulador;
%
%ProbeInfo = vetor binário que indica a distribuicao/posicao de probes ao
%longo do quadro transmitido ('1' indica que o simbolo naquela posicao e
%conhecido (probe), enquanto '0' indica que o simbolo naquela posicao e
%desconhecido no receptor). Para interleaving curto, as primeiras 1440
%posicoes correspondem ao preambulo (para interleaving longo, este valor e de
%11520). Apos estes valores, as posicoes seguintes correspondem a
%intercalacoes de quadros de dados com dados de probes de treinamento. Para
%a forma de onde 2400bps, o tamanho dos quadros de dados e de 32 simbolos,
%enquanto o dos de probes e de 16 simbolos.
%
%MultiplexOut = saida do transmissor. Simbolos PSK, complexos, de saida
%do modulador, incluindo preambulo, probes e dados. Entrada do modulo
%formatador de pulso (segundo a norma, raiz de coseno levantado). Seu
%tamanho e o mesmo do vetor ProbeInfo;
TxOut            = mil110CTransmissor(mil110C, varMil110C.nFrames, varMil110C.seed);

%mil110CFiltroTransmissor = simula o siltro de saida do transmissor
%(formatador de pulso). Retorna um struct 'ShappingOut' com as seguintes
%saidas:
%
%filter = filtro raiz de coseno levantado utilizado na formatacao dos
%pulsos. Seu tamanho se da pela relacao '2 x pulseDelay x oversamplingFactor + 1';
%
%out = dados de saida do formatador de pulso. Pulsos ocupando
%largura de banda (1 + rolloffFactor) x 1200 (banda base). Seu
%tamanho corresponde ao tamanho do vetor 'MultiplexOut' multiplicado pelo
%fator de superamostragem (oversamplingFactor), acrescido ainda das
%amostras de retardo (pulseDelay x oversamplingFactor) introduzidas pelo
%filtro no começo e no fim do vetor;
ShappingOut      = mil110CFiltroTransmissor(TxOut.MultiplexOut, varShapping);

%mil110CCanalWatterson = simula o canal HF ionosferico, segundo o modelo de
%canal proposto por (Watterson, 1970). Retorna um struct 'WattersonOut' com
%as seguintes saidas:
%
%raios = matriz com 2 colunas, cada uma representando um dos raios 'Watterson'
%ao longo da transmissao do sinal. Seu tamanho e o mesmo do vetor
%'ShappingOut.out';
%
%RER = atraso entre os raios, em segundos;
%
%ED = espalhamento Doppler dos raios, em Hz;
%
%WattersonOutAwgnIn = sinal convoluido com os raios e, portanto, com presenca
%de ISI. Seu tamanho corresponde ao do vetor 'ShappingOut.out', acrescido
%do atraso (em amostras) entre os dois raios do canal Watterson (RER x 2400
% x oversamplingFactor);
%
%noise = ruido AWGN adicionado ao sinal 'WattersonOutAwgnIn'. Seu tamanho e
%o mesmo do vetor 'WattersonOutAwgnIn';
%
%AwgnOut = dados de saida do emulador de canal Watterson.
%Corresponde ao sinal com presenca de ISI e contaminado com AWGN. Seu
%tamanho e o mesmo do vetor 'WattersonOutAwgnIn';
WattersonOut     = mil110CCanalWatterson(ShappingOut.out, varWatterson, varShapping);

%mil110CFiltroCasado = simula o filtro de recepcao. Para este simulador
%basico proposto, trata-se de um filtro casado ao filtro formatador de
%pulso. Retorna um struct 'MatchedFilterOut' com as seguintes saidas:
%
%filter = filtro utilizado na filtragem do sinal de recepcao. Neste caso,
%ele e semelhante ao 'ShappingOut.filter';
%
%out = dados de saida do filtro de recepcao. Neste caso, corresponde ao sinal
%confinado novamente na largura de banda (1 + rolloffFactor) x 1200
%(banda base). Seu tamanho corresponde ao tamanho do vetor 'AwgnOut'
%acrescido das amostras de retardo introduzidas pelo filtro no começo e no
%final do vetor;
MatchedFilterOut = mil110CFiltroCasado(WattersonOut.AwgnOut, varShapping);
%MatchedFilterOut = mil110CFiltroCasado(ShappingOut.out, varShapping);

%mil110CSincronismoQuadro = simula um sincronizador de quadro simples. O modelo
%Watterson para o canal HF é um filtro FIR com 2 componentes principais.
%Para recuperar o sinal transmitido, o receptor precisa sincronizar a cadeia
%de recepção em um desses dois componentes. Esse papel é realizado por alguma
%técnica de sincronismo de quadro, no início da recepção do sinal. Para este
%trabalho, considerou-se que o sincronismo de quadro encontra o componente de
%maior módulo e sincroniza a cadeia de recepção em torno deste cursor. O outro
%componente passa a ser um pós-cursor (caso o primeiro componente seja sincronizado)
%ou um pré-cursor (caso o segundo o seja). Este sincronizador simples
%retorna um numero, que e uma estimativa da amostra onde se identifica o
%comeco da transmissao. Na ausencia de canal, ele retornara o valor
%correspondente às amostras de retardo dos filtros de transmissao e de
%recepcao, mais um;
FrameStartSample = mil110CSincronismoQuadro(MatchedFilterOut.out, mil110C, varShapping);

%mil110CSincronismoSimbolo = simula um sincronizador de simbolo (ponto de
%amostragem) simples. Neste caso, ele apenas recebe o ponto de inicio da
%transmissao e amostra o sinal de saida do filtro de recepcao em intervalos
%de 'varShapping.oversamplingFactor' amostras. Retorna um struct 'SyncSymbolOut'
%com a seguinte saida:
%
%out = dados de saida do sincronizador de simbolo. Corresponde a um vetor de
%simbolos PSK, complexos. Na ausencia de canal, deve ser identico ao vetor
%'TxOut.MultiplexOut'. Na presenca de canal, sera uma versao distorcida de
%'MultiplexOut', e o seu tamanho correspondera ao de 'MultiplexOut'
%acrescido de alguns simbolos no final, correspondentes ao espalhamento de
%atraso provocado pelo canal;
SyncSymbolOut    = mil110CSincronismoSimbolo(MatchedFilterOut.out, FrameStartSample, varShapping);

%mil110CDemultiplexador = simula o demultiplexador do receptor. Sua funcao
%e separar os simbolos PSK de dados do preambulo e dos probes, além de fazer
%o descrambling (desembaralhamento) dos simbolos. Retorna um struct
%'DemultiplexOut' com a seguinte saida:
%
%out = vetor com simbolos PSK, complexos, de dados. Na ausencia de canal,
%deve ser identico ao vetor 'TxOut.ComplexPSKSymbols'. Seu tamanho e
%identico ao de 'ComplexPSKSymbols';
DemultiplexOut   = mil110CDemultiplexador(SyncSymbolOut.out, mil110C);

%mil110CDemodulador = simula o demodulador PSK e o demapeador Gray no
%receptor. Retorna um struct 'DemodulatorOut' com as seguintes saidas:
%
%DemodOutDemapIn = simbolos PSK, inteiros, de saida do demodulador e de
%entrada do demapeador Gray. Seu tamanho e identico ao do vetor
%'DemultiplexOut.out';
%
%DemapOut = 'hard' bits de saida do demapeador Gray, ainda sob efeito de
%entrelacamento (interleaving) e codificacao convolucional. Na ausencia de
%canal, deve ser igual ao vetor 'TxOut.InterOutMapIn'. Seu tamanho e
%identico ao de 'InterOutMapIn';
DemodulatorOut   = mil110CDemodulador(DemultiplexOut.out, mil110C);

%mil110CDeinterleaver = simula o deinterleaver no receptor. Retorna um
%struct com a seguinte saida:
%
%out = vetor com bits ('hard', quando a entrada for 'hard' bits; ou 'soft',
%quando a entrada for 'soft' bits) desentrelacados, mas ainda
%codificados. Na ausencia de canal e quando consistir de 'hard' bits,
%deve ser identico ao vetor 'TxOut.CodOutInterIn'. Seu tamanho e identico
%ao de 'CodOutInterIn';
DeinterleaverOut = mil110CDeinterleaver(DemodulatorOut.DemapOut, mil110C);

ber_demod = biterr(TxOut.CodOutInterIn, DeinterleaverOut.out)/length(TxOut.CodOutInterIn);
